#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
mms_config.py

@author : Louis RICHARD
"""

CONFIG = {"local_data_dir": "/Users/louisr/Documents/PhD/solo",
          'mirror_data_dir': None,  # e.g., '/Volumes/data_network/data/mms'
          'debug_mode': False,
          'download_only': False,
          'no_download': False}